// Planet.cpp: implementation of the Planet class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Solar.h"
#include "Planet.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Planet::Planet()
{
	// This planet feels the effects of all the other planets
	// in the solar system:
	m_PlanetCount++;
	m_Color = RGB(255,255,255);
}

// The copy ctor:
Planet::Planet(const Planet &copy)
{
	m_Color = copy.m_Color;
	m_DispPos = copy.m_DispPos;
	m_Mass = copy.m_Mass;
	m_Rect = copy.m_Rect;
	m_xLocation = copy.m_xLocation;
	m_xSpeed = copy.m_xSpeed;
	m_yLocation = copy.m_yLocation;
	m_ySpeed = copy.m_ySpeed;
}

// The (=) assignment:
const Planet& Planet::operator=(const Planet &right)
{
	if (&right == this)
		return *this;

	m_Color = right.m_Color;
	m_DispPos = right.m_DispPos;
	m_Mass = right.m_Mass;
	m_Rect = right.m_Rect;
	m_xLocation = right.m_xLocation;
	m_xSpeed = right.m_xSpeed;
	m_yLocation = right.m_yLocation;
	m_ySpeed = right.m_ySpeed;
	
	return *this;
}


Planet::~Planet()
{

}

void Planet::setMass(double mass)
{
	// Mass is in kilograms:
	m_Mass = mass;
	MakeRect();		// if mass changes, rect changes
}

void Planet::setPos(CPoint point)
{
	m_DispPos = point;
	MakeRect();		// if position changes, rect changes

	// Calculate an actual spatial location:
	// (Scale: 1 pixel = 1 billion meters)
	m_xLocation = point.x * 1.0e9;
	m_yLocation = point.y * 1.0e9;
}

CPoint Planet::getDispPos()
{
	return m_DispPos;
}

double Planet::getMass()
{
	return m_Mass;
}

COLORREF Planet::getColor()
{
	return m_Color;
}

void Planet::setColor(COLORREF color)
{
	m_Color = color;
}

CRect Planet::getRect()
{
	return m_Rect;
}

void Planet::MakeRect()
{
	int size = 6;
	if (m_Mass >= 1e34)
		size+=8;
	else if (m_Mass >= 1e33)
		size+=7;
	else if (m_Mass >= 1e32)
		size+=6;
	else if (m_Mass >= 1e31)
		size+=5;
	else if (m_Mass >= 1e30)
		size+=4;
	else if (m_Mass >= 1e29)
		size+=3;
	else if (m_Mass >= 1e28)
		size+=2;
	else if (m_Mass >= 1e27)
		size++;
	m_Rect = CRect(m_DispPos.x-size, m_DispPos.y-size,
		m_DispPos.x+size, m_DispPos.y+size);
}

void Planet::Tick(double step)
{
	// Make time pass.  Update the position:
	// Position = Speed * time;
	m_xLocation = m_xLocation + (m_xSpeed * step);
	m_yLocation = m_yLocation + (m_ySpeed * step);

	// Translate the location to a display position:
	// Scale: 1 pixel = 1 billion meters
	m_DispPos.x = (int)(m_xLocation / 1.0e9);
	m_DispPos.y = (int)(m_yLocation / 1.0e9);
	MakeRect();
}

void Planet::setXSpeed(double velocity)
{
	m_xSpeed = velocity;
}

void Planet::setYSpeed(double velocity)
{
	m_ySpeed = velocity;
}

void Planet::addXSpeed(double velocity)
{
	m_xSpeed += velocity;
}

void Planet::addYSpeed(double velocity)
{
	m_ySpeed += velocity;
}

double Planet::getXLoc()
{
	return m_xLocation;
}

double Planet::getYLoc()
{
	return m_yLocation;
}

double Planet::getXSpeed()
{
	return m_xSpeed;
}

double Planet::getYSpeed()
{
	return m_ySpeed;
}
