// PropDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Solar.h"
#include "SolarFrm.h"
#include "PropDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// PropDlg dialog


PropDlg::PropDlg(CWnd* pParent /*=NULL*/)
	: CDialog(PropDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(PropDlg)
	m_csMassDisplay = _T("");
	m_csXVelocity = _T("");
	m_csYVelocity = _T("");
	m_csXPos = _T("");
	m_csYPos = _T("");
	//}}AFX_DATA_INIT
	m_Mass = 1.0e25;		// A small planet
	m_bRemovable = FALSE;
}


void PropDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(PropDlg)
	DDX_Control(pDX, IDC_YVELOCITY, m_YVelCtrl);
	DDX_Control(pDX, IDC_XVELOCITY, m_XVelCtrl);
	DDX_Control(pDX, IDC_YVEL_SPIN, m_YSpin);
	DDX_Control(pDX, IDC_XVEL_SPIN, m_XSpin);
	DDX_Control(pDX, IDC_REMOVE, m_RemoveButton);
	DDX_Control(pDX, IDC_MASS_SLIDER, m_MassSlider);
	DDX_Text(pDX, IDC_MASS, m_csMassDisplay);
	DDX_Text(pDX, IDC_XVELOCITY, m_csXVelocity);
	DDX_Text(pDX, IDC_YVELOCITY, m_csYVelocity);
	DDX_Text(pDX, IDC_XPOS, m_csXPos);
	DDX_Text(pDX, IDC_YPOS, m_csYPos);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(PropDlg, CDialog)
	//{{AFX_MSG_MAP(PropDlg)
	ON_BN_CLICKED(IDC_COLOR, OnColor)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	ON_WM_HSCROLL()
	ON_WM_PAINT()
	ON_WM_VSCROLL()
	ON_EN_CHANGE(IDC_XVELOCITY, OnChangeXvelocity)
	ON_EN_CHANGE(IDC_YVELOCITY, OnChangeYvelocity)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// PropDlg message handlers

void PropDlg::OnColor() 
{
	CColorDialog dlg;
	if (dlg.DoModal() == IDOK)
	{
		m_Color = dlg.GetColor();
		CRect colorrect(6, 236, 22, 258);
		InvalidateRect(colorrect);
	}
}

void PropDlg::OnRemove() 
{
	((CSolarFrm *)GetParent())->RemoveBody(m_BodyNum);
	EndDialog(IDCANCEL);
}

void PropDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int pos = m_MassSlider.GetPos();
	// Convert the position to a mass:
	m_Mass = ((pos%9)+1) * pow(10, ((pos/9) + 25));
	m_csMassDisplay.Format("%.2e", m_Mass);
	UpdateData(FALSE);

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

BOOL PropDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_csTitle.Format("Properties: body %d", m_BodyNum + 1);
	
	// Put up the Mass Slider:
	m_MassSlider.SetRange(0, 90, FALSE);
	m_XSpin.SetRange(0, 1);
	m_XSpin.SetPos(0);
	m_YSpin.SetRange(0, 1);
	m_YSpin.SetPos(0);
	for (int pos=0; pos<=100; pos++)
	{
		if ((m_Mass - (((pos%9)+1) * pow(10, ((pos/9) + 25)))) < 100)
		{
			m_MassSlider.SetPos(pos);
			m_csMassDisplay.Format("%.2e", m_Mass);
			break;
		}
	}
	SetWindowText(m_csTitle);
	m_RemoveButton.EnableWindow(m_bRemovable);
	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void PropDlg::SetPosition(CPoint point)
{
	m_csXPos.Format("%de+009", point.x);
	m_csYPos.Format("%de+009", point.y);	
}

void PropDlg::SetVelocity(CPoint start, CPoint end)
{
	m_XVel = (end.x - start.x) * 1000.0;
	m_csXVelocity.Format("%d", (int)m_XVel);
	m_YVel = (end.y - start.y) * 1000.0;
	m_csYVelocity.Format("%d", (int)m_YVel);
}

void PropDlg::SetBodyNum(int num)
{
	m_BodyNum = num;
}

void PropDlg::SetMass(double mass)
{
	m_Mass = mass;
}


void PropDlg::SetSpeed(double xspeed, double yspeed)
{
	// pass on the speed to the planets:
	m_XVel = xspeed;
	m_YVel = yspeed;
}

void PropDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CBrush newbrush(m_Color);
	CBrush *oldbrush = dc.SelectObject(&newbrush);
	dc.Rectangle(6, 236, 22, 258);
	dc.SelectObject(oldbrush);
	// Do not call CDialog::OnPaint() for painting messages
}

void PropDlg::SetColor(COLORREF color)
{
	m_Color = color;
}

void PropDlg::SetRemovable(BOOL removable)
{
	m_bRemovable = removable;
}

void PropDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CString csIndicator;
	if (pScrollBar->GetDlgCtrlID() == IDC_XVEL_SPIN)
	{
		if (nSBCode == SB_THUMBPOSITION)	// Add velocity
		{
			if (nPos > 0)			// Going up
				m_XVel += 1000.0;
			else					// Going down
				m_XVel -= 1000.0;
			m_csXVelocity.Format("%.1f", m_XVel);
		}
	}
	else	// ICC_YVEL_SPIN
	{
		if (nSBCode == SB_THUMBPOSITION)	// Add velocity
		{
			if (nPos > 0)			// Going up
				m_YVel += 1000.0;
			else					// Going down
				m_YVel -= 1000.0;
			m_csYVelocity.Format("%.1f", m_YVel);
		}
	}
	UpdateData(FALSE);
	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void PropDlg::OnChangeXvelocity() 
{
	// User typed in a new X Velocity:
	m_XVelCtrl.GetWindowText(m_csXVelocity);
	m_XVel = atoi(m_csXVelocity);
}

void PropDlg::OnChangeYvelocity() 
{
	// User typed in a new Y Velocity:
	m_YVelCtrl.GetWindowText(m_csYVelocity);
	m_YVel = atoi(m_csYVelocity);
}
