// SolarFrm.cpp : implementation file
//

//
// Guy T. Schafer, guy@professorguy.org
// Project 2 (Homework 8), CSCI E-141
// November 30, 1999
//
// Solar System Builder
// This program allows the user to place gravitational bodies
// onto a 2D space and let them interact.
// Clicking sets a body, Dragging sets the velocity vector.
// User can manipulate the mass of the object.
//
// All calculations are performed in meters per second (m/s).
// The simulation speed can be adjusted for best accuracy
// (time ticks in 1 second increments) or best speed (time
// ticks in 100000 second increments).  Velocity is recalculated
// at the begining of each time increment.
//

#include "stdafx.h"
#include "Solar.h"
#include "SolarFrm.h"
#include "PropDlg.h"
#include "HelpDlg.h"
#include "AboutDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSolarFrm

CSolarFrm::CSolarFrm()
	: m_nBodyCount(0), m_bDragging(FALSE),
	m_Time(0.0), m_TimeStep(100.0), m_G(G)
{
}

CSolarFrm::~CSolarFrm()
{
}


BEGIN_MESSAGE_MAP(CSolarFrm, CFrameWnd)
	//{{AFX_MSG_MAP(CSolarFrm)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_SIZE()
	ON_WM_RBUTTONDOWN()
	ON_WM_VSCROLL()
	ON_COMMAND(ID_HELP_INSTRUCTIONS, OnHelpInstructions)
	ON_COMMAND(ID_HELP_ABOUTSSB, OnHelpAbout)
	ON_COMMAND(ID_FILE_EXIT, OnFileExit)
	ON_UPDATE_COMMAND_UI(ID_CONTROL_GO, OnUpdateControlGo)
	ON_COMMAND(ID_CONTROL_GO, OnGo)
	ON_COMMAND(ID_CONTROL_STOP, OnStop)
	ON_UPDATE_COMMAND_UI(ID_CONTROL_STOP, OnUpdateControlStop)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BTN_GO, OnGo)
	ON_BN_CLICKED(IDC_BTN_STOP, OnStop)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_POS,
	ID_INDICATOR_TIME,
	ID_INDICATOR_STEP,
	ID_INDICATOR_GRAV,
};


/////////////////////////////////////////////////////////////////////////////
// CSolarFrm message handlers

int CSolarFrm::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_wndStatusBar.Create(this);
	m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT));

	// Pick a fake rectangle to stuff all the controls into.
	// They will all be resized during WM_SIZE at initialization
	CRect r(0,0,1,1);

	// Put up the fake panel frame:
	m_ControlGB.Create("", WS_CHILD|WS_VISIBLE|BS_GROUPBOX,
					r, this, IDC_STATIC);

	// Put up the Buttons:
	m_GoButton.Create("Go", WS_CHILD|WS_VISIBLE|WS_DISABLED|BS_PUSHBUTTON,
					r, this, IDC_BTN_GO);
	m_StopButton.Create("Stop", WS_CHILD|WS_VISIBLE|WS_DISABLED|BS_PUSHBUTTON,
					r, this, IDC_BTN_STOP);

	// Put up the Time Step slider:
	m_TimeStepGB.Create("delta t", WS_CHILD|WS_VISIBLE|BS_GROUPBOX,
					r, this, IDC_STATIC);
	m_TimeStepSldr.Create(WS_CHILD|WS_VISIBLE|TBS_VERT|TBS_RIGHT|TBS_AUTOTICKS,
					r, this, IDC_SB_TIMESTEP);
	m_TimeStepSldr.SetRange(0, 5);
	m_TimeStepSldr.SetPos(3);

	// Put up the Gravity slider:
	m_GravityGB.Create("G", WS_CHILD|WS_VISIBLE|BS_GROUPBOX,
					r, this, IDC_STATIC);
	m_GravitySldr.Create(WS_CHILD|WS_VISIBLE|TBS_VERT|TBS_NOTICKS,
					r, this, IDC_SB_GRAVITY);
	m_GravitySldr.SetRange(50, 150);
	m_GravitySldr.SetPos(100);
	m_G1LabelBitmap.Create(NULL, WS_CHILD|WS_VISIBLE|SS_BITMAP,
					r, this, IDC_STATIC);
	m_G2LabelBitmap.Create(NULL, WS_CHILD|WS_VISIBLE|SS_BITMAP,
					r, this, IDC_STATIC);
	m_G3LabelBitmap.Create(NULL, WS_CHILD|WS_VISIBLE|SS_BITMAP,
					r, this, IDC_STATIC);
	m_TSLabelBitmap.Create(NULL, WS_CHILD|WS_VISIBLE|SS_BITMAP,
					r, this, IDC_STATIC);
	// Bitmap loading:
	m_Bitmap.Detach();
	m_Bitmap.LoadBitmap(IDB_G1);
	m_G1LabelBitmap.SetBitmap(m_Bitmap);
	m_Bitmap.Detach();
	m_Bitmap.LoadBitmap(IDB_G2);
	m_G2LabelBitmap.SetBitmap(m_Bitmap);
	m_Bitmap.Detach();
	m_Bitmap.LoadBitmap(IDB_G3);
	m_G3LabelBitmap.SetBitmap(m_Bitmap);
	m_Bitmap.Detach();
	m_Bitmap.LoadBitmap(IDB_TS);
	m_TSLabelBitmap.SetBitmap(m_Bitmap);

	// Initialize the indicator Panes:
	CString csInit("t: 0.000000e+000 s");
	m_wndStatusBar.SetPaneText(2, csInit);
	csInit.Format("       100 s", m_TimeStep);
	m_wndStatusBar.SetPaneText(3, csInit);
	csInit.Format("%.4e", m_G);
	m_wndStatusBar.SetPaneText(4, csInit);

	// Disable menu commands:
	m_bGoInit = FALSE;
	m_bStopInit = FALSE;
	
	return 0;
}

void CSolarFrm::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CRect r;
	GetViewRect(&r);
	dc.FillSolidRect(r, RGB(0,0,0));

	// Draw any visible vectors (during mouse drags):
	if (m_bDragging)
	{
		CPen *p = (CPen *)(dc.SelectStockObject(WHITE_PEN));
		dc.MoveTo(m_DownPoint.x, m_DownPoint.y);
		dc.LineTo(m_CurPoint.x, m_CurPoint.y);
		dc.SelectObject(p);
	}
	// Draw the planets:
	CRect PlanetRect;
	CBrush ob(m_Bodies[0].getColor());
	CBrush *oldbrush = dc.SelectObject(&ob);
	for (int i=0; i<m_nBodyCount; i++)
	{
		PlanetRect = m_Bodies[i].getRect();
		CBrush pb(m_Bodies[i].getColor());
		dc.SelectObject(&pb);
		dc.Ellipse(PlanetRect);
		dc.SelectStockObject(BLACK_BRUSH);
		dc.Ellipse(PlanetRect.left+3, PlanetRect.top+3,
				PlanetRect.right-3, PlanetRect.bottom-3);
		dc.SetPixel(m_Bodies[i].getDispPos(), RGB(255,255,255));
	}

	// Put up the fake control panel:
	//dc.SelectStockObject(LTGRAY_BRUSH);
	m_ControlRect = CRect(r.right-141, r.bottom-273,
				r.right-6, r.bottom-6); 
	dc.SelectStockObject(LTGRAY_BRUSH);
	dc.Rectangle(m_ControlRect);
	// Go back to your homes, there's nothing to see here:
	dc.SelectObject(oldbrush);
}

void CSolarFrm::GetViewRect(LPRECT r)
{
	RepositionBars(0, 0xffff, AFX_IDW_PANE_FIRST, reposQuery, r);
}

BOOL CSolarFrm::PreCreateWindow(CREATESTRUCT& cs) 
{
//Remove the next line if you want a resizable window
	cs.style &= (~WS_THICKFRAME);
//This sets the width and height of the window
	cs.cx = 600;
	cs.cy = 450;
//This sets the title (in the caption bar)
	cs.lpszName = "Solar System Builder v1.0";
	return CFrameWnd::PreCreateWindow(cs);
}

void CSolarFrm::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (m_nBodyCount >= MAX_BODIES)
	{
		CString csMsg;
		csMsg.Format("Cannot add bodies (%d max)", MAX_BODIES);
		SetMessageText(csMsg);
		return;
	}
	// Don't put planets behind the control panel:
	// (otherwise, when the user accidentally misses
	// the controls, we put up planets)
	if (m_ControlRect.PtInRect(point))
	{
		m_bIgnoreUp = TRUE;
		return;
	}
	m_bIgnoreUp = FALSE;

	// Everybody remember where we parked:
	m_DownPoint = point;
	m_Bodies[m_nBodyCount].setPos(m_DownPoint);

	// Draw the body:
	m_bDragging = TRUE;

	CFrameWnd::OnLButtonDown(nFlags, point);
}

void CSolarFrm::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_nBodyCount >= MAX_BODIES)
	{
		// They already got the error on button down,
		// let's not rub it in:
		SetMessageText("Ready");
		return;
	}
	if (m_bIgnoreUp)
		return;

	CString csIndicator;
	m_bDragging = FALSE;

	// Pop up a properties dialog box:
	PropDlg dlg(this);
	dlg.SetPosition(m_DownPoint);
	dlg.SetVelocity(m_DownPoint, point);
	dlg.SetBodyNum(m_nBodyCount);
	dlg.SetColor(RGB(255,255,255));	// default color
	dlg.SetRemovable(FALSE);		// No planet yet.
	if (dlg.DoModal() == IDOK)
	{
		m_Bodies[m_nBodyCount].setMass(dlg.m_Mass);
		m_Bodies[m_nBodyCount].setColor(dlg.m_Color);
		int l = min(m_DownPoint.x, m_CurPoint.x) - 15;
		int r = max(m_DownPoint.x, m_CurPoint.x) + 15;
		int t = min(m_DownPoint.y, m_CurPoint.y) - 15;
		int b = max(m_DownPoint.y, m_CurPoint.y) + 15;
		m_CurPoint = point;
		if (l < 0) l = 0;
		if (t < 0) t = 0;
		CRect rect(l, t, r, b);
		InvalidateRect(rect, FALSE);
		
		// Set the velocity vector:
		m_Bodies[m_nBodyCount].setXSpeed(dlg.m_XVel);
		m_Bodies[m_nBodyCount].setYSpeed(dlg.m_YVel);
		
		// Now a complete planet has been added:
		if (m_nBodyCount == 0)	// the first time:
		{
			m_GoButton.EnableWindow(TRUE);
			m_bGoInit = TRUE;
		}
		m_nBodyCount++;
	}
	CFrameWnd::OnLButtonUp(nFlags, point);
}

void CSolarFrm::OnMouseMove(UINT nFlags, CPoint point) 
{
	CString csIndicator;
	csIndicator.Format("x: %de+009  y:%de+009",
		point.x, point.y);
	m_wndStatusBar.SetPaneText(1, csIndicator);

	if (m_bDragging)
	{
		int l = min(m_DownPoint.x, m_CurPoint.x) - 10;
		int r = max(m_DownPoint.x, m_CurPoint.x) + 10;
		int t = min(m_DownPoint.y, m_CurPoint.y) - 10;
		int b = max(m_DownPoint.y, m_CurPoint.y) + 10;
		// Remember this point so OnPaint() can draw the vector:
		m_CurPoint = point;
		if (l < 0) l = 0;
		if (t < 0) t = 0;
		// Draw the vector:
		CRect rect(l, t, r, b);
		InvalidateRect(rect, FALSE);
	}
	CFrameWnd::OnMouseMove(nFlags, point);
}

void CSolarFrm::OnGo()
{
	SetMessageText("Running");
	// Set the button and menu states:
	m_GoButton.EnableWindow(FALSE);
	m_StopButton.EnableWindow(TRUE);
	m_bGoInit = FALSE;
	m_bStopInit = TRUE;

	// And god created the heavens and the earth:

	CString cs;
	int i, j;
	CRect oldrect;
	// god is immortal:
	m_bRunLoop = TRUE;
	while (m_bRunLoop)
	{
		// OK, here's the meat of this program. The idea
		// is each planet has a current velocity.  Now,
		// let's modify each of the planet's velocities
		// according to the gravitational effects of all
		// the other planets.
		// The velocities are changed according to the
		// equation first worked out by Sir Isaac Newton:
		// F = G * ((M1 * M2) / r*r)
		// where M1 and M2 are the masses involved,
		// r is the distance between them,
		// and G is the gravitational constant of the universe.
		// To solve for the velocity that would be imparted
		// to a body (M1) under influence of another (M2),
		// v = ((G * M2) / r*r) * timestep
		// (Notice that our mass doesn't matter--all bodies
		// fall at the same rate.  Thanks Gallileo.)
		// By simply adding up all the velocities due to masses
		// and the current velocity, we get a new velocity.
		// Then the new position is simply:
		// p = v * timestep;

		// Go through and add up the velocities (i is us, j is them):
		double xdist, ydist, distsq, dist;
		double velx, vely, vel;
		for (i=0; i<m_nBodyCount; i++)
		{
			for (j=0; j<m_nBodyCount; j++)
			{
				if (i == j)		// Don't affect yourself
					continue;
				// Measure the distance between the bodies:
				xdist = m_Bodies[j].getXLoc() - m_Bodies[i].getXLoc();
				ydist = m_Bodies[j].getYLoc() - m_Bodies[i].getYLoc();
				distsq = (xdist * xdist) + (ydist * ydist);
				if (distsq < 1.0)	// don't divide by 0
					distsq = 1.0;		// 0 meters ~= 1 meter
				vel = m_TimeStep * m_G * m_Bodies[j].getMass() / distsq;
				// Don't need trig if we use similar triangles:
				dist = sqrt(distsq);
				velx = xdist * vel / dist;
				vely = ydist * vel / dist;
				m_Bodies[i].addXSpeed(velx);
				m_Bodies[i].addYSpeed(vely);
			}
		}
		// The entire reason I chose to take on this project
		// is so I could put in the following comment:

		// MOVE THE PLANETS:
		for (i=0; i<m_nBodyCount; i++)
		{
			// Remember where it was:
			oldrect = m_Bodies[i].getRect();
			m_Bodies[i].Tick(m_TimeStep);	// how much time passed

			if (oldrect != m_Bodies[i].getRect())
			{
				InvalidateRect(oldrect, FALSE);
				// Draw where it is:
				InvalidateRect(m_Bodies[i].getRect(), FALSE);
			}
		}
		// Adjust the time and tell the user:
		m_Time += m_TimeStep;
		cs.Format("t: %e s", m_Time);
		m_wndStatusBar.SetPaneText(2, cs);

		// Even god must obey the user:
		if(!PeekAndPump())
			return;
	}
}

void CSolarFrm::OnStop()
{
	SetMessageText("Ready");
	// Set the button and menu states:
	m_GoButton.EnableWindow(TRUE);
	m_StopButton.EnableWindow(FALSE);
	m_bGoInit = TRUE;
	m_bStopInit = FALSE;
	// Bail out of the Go Loop:
	m_bRunLoop = FALSE;
}

void CSolarFrm::OnSize(UINT nType, int cx, int cy) 
{
	CFrameWnd::OnSize(nType, cx, cy);
	
	CRect r;
	GetViewRect(&r);

	// Move the fake control panel:
	CRect rButton(r.right-140, r.bottom-280, r.right-5, r.bottom-5);
	m_ControlGB.MoveWindow(rButton);

	// Move the buttons:
	rButton = CRect(r.right-130, r.bottom-265, r.right-80, r.bottom-240);
	m_GoButton.MoveWindow(rButton);
	rButton = CRect(r.right-65, r.bottom-265, r.right-15, r.bottom-240);
	m_StopButton.MoveWindow(rButton);

	// Move the Time Step slider:
	rButton = CRect(r.right-135, r.bottom-230, r.right-75, r.bottom-10);
	m_TimeStepGB.MoveWindow(rButton);
	rButton = CRect(r.right-132, r.bottom-210, r.right-105, r.bottom-20);
	m_TimeStepSldr.MoveWindow(rButton);
	rButton = CRect(r.right-102, r.bottom-190, r.right-87, r.bottom-40);
	m_TSLabelBitmap.MoveWindow(rButton);

	// Move the Gravity slider:
	rButton = CRect(r.right-70, r.bottom-230, r.right-10, r.bottom-10);
	m_GravityGB.MoveWindow(rButton);
	rButton = CRect(r.right-67, r.bottom-210, r.right-40, r.bottom-20);
	m_GravitySldr.MoveWindow(rButton);
	rButton = CRect(r.right-40, r.bottom-203, r.right-15, r.bottom-193);
	m_G1LabelBitmap.MoveWindow(rButton);
	rButton = CRect(r.right-40, r.bottom-121, r.right-15, r.bottom-111);
	m_G2LabelBitmap.MoveWindow(rButton);
	rButton = CRect(r.right-40, r.bottom-41, r.right-15, r.bottom-31);
	m_G3LabelBitmap.MoveWindow(rButton);
}

BOOL CSolarFrm::PeekAndPump()
{
	MSG msg;
	while(::PeekMessage(&msg, 0, 0, 0, PM_NOREMOVE))
	{
		if(!AfxGetApp()->PumpMessage())
		{
			::PostQuitMessage(0);
			return FALSE;
		}
	}
	LONG lIdle = 0;
	while(AfxGetApp()->OnIdle(lIdle++));
	return TRUE;
}

void CSolarFrm::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CString csIndicator;
	// See if I'm near a planet:
	for (int i=0; i<m_nBodyCount; i++)
	{
		if (m_Bodies[i].getRect().PtInRect(point))
		{
			// Make a properties dialog box:
			PropDlg dlg(this);
			// Initialize the title bar:
			dlg.SetBodyNum(i);
			// Initialize the position display boxes:
			dlg.SetPosition(m_Bodies[i].getDispPos());
			// Initialize the velocity edit boxes:
			csIndicator.Format("%d", (int)(m_Bodies[i].getXSpeed()));
			dlg.m_csXVelocity = csIndicator;
			csIndicator.Format("%d", (int)(m_Bodies[i].getYSpeed()));
			dlg.m_csYVelocity = csIndicator;
			dlg.SetSpeed(m_Bodies[i].getXSpeed(), m_Bodies[i].getYSpeed());
			dlg.m_Color = m_Bodies[i].getColor();
			// Initialize the mass slider position:
			dlg.SetMass(m_Bodies[i].getMass());
			// Set color:
			dlg.SetColor(m_Bodies[i].getColor());
			dlg.SetRemovable(TRUE);		// Planet exists already.

			// If user adjusts mass down, old rect might not redraw:
			CRect oldrect = m_Bodies[i].getRect();
			// Pop it up:
			if (dlg.DoModal() == IDOK)
			{
				m_Bodies[i].setMass(dlg.m_Mass);
				m_Bodies[i].setColor(dlg.m_Color);
				// one of these is the bigger, who cares which:
				InvalidateRect(oldrect, FALSE);
				InvalidateRect(m_Bodies[i].getRect());
				
				// Set the velocity vector:
				m_Bodies[i].setXSpeed(dlg.m_XVel);
				m_Bodies[i].setYSpeed(dlg.m_YVel);
			}
			return;
		}
	}
	CFrameWnd::OnRButtonDown(nFlags, point);
}

void CSolarFrm::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CString csIndicator;
	if (pScrollBar->GetDlgCtrlID() == IDC_SB_TIMESTEP)
	{
		// Adjust the timestep:
		int pos = ((CSliderCtrl *)pScrollBar)->GetPos();
		// Make the bottom small, top big
		pos = ((CSliderCtrl *)pScrollBar)->GetRangeMax() - pos;
		m_TimeStep = pow(10.0, (double)pos);
		csIndicator.Format("%10d s", (int)m_TimeStep);
		m_wndStatusBar.SetPaneText(3, csIndicator);
	}
	// User is changing the gravitational constant
	// of the universe again:
	else
	{
		int pos = ((CSliderCtrl *)pScrollBar)->GetPos();
		pos = 200 - pos;	// 150 to 50 instead of 50 to 150
		m_G = G * .01 * pos;
		csIndicator.Format("%.4e", m_G);
		m_wndStatusBar.SetPaneText(4, csIndicator);
	}
	CFrameWnd::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CSolarFrm::OnFileExit() 
{
	DestroyWindow();
}
void CSolarFrm::PostNcDestroy() 
{
	delete this;
}

void CSolarFrm::OnHelpInstructions() 
{
	// Pop up an instructions dialog box:
	HelpDlg dlg(this);
	dlg.DoModal();
}

void CSolarFrm::OnHelpAbout() 
{
	// Pop up an about box:
	AboutDlg dlg(this);
	dlg.DoModal();
}

void CSolarFrm::OnUpdateControlGo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bGoInit);
}

void CSolarFrm::OnUpdateControlStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bStopInit);
}

void CSolarFrm::RemoveBody(int num)
{
	// Get rid of planet num:
	CRect ByeRect = m_Bodies[num].getRect();

	// Overwrite that planet:
	m_nBodyCount--;
	for (int i=num; i<m_nBodyCount; i++)
		m_Bodies[i] = m_Bodies[i+1];

	// Make it go away:
	InvalidateRect(ByeRect, FALSE);
}
